<?php

namespace Drupal\cmlapi\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\cmlapi\Xml\XmlObject;
use Drupal\cmlapi\Xml\CatalogParcer;
use Symfony\Component\Yaml\Yaml;

/**
 * Controller routines for page example routes.
 */
class Catalog extends ControllerBase {

  /**
   * Page tree.
   */
  public function page($cml) {
    $result = '';
    $data = $this->data($cml);
    if ($data) {
      $result .= '<div id="jstree">';
      $result .= $this->renderGroups($data['group'], TRUE);
      $result .= '</div>';
      $feature = Yaml::dump(self::prepareFeature($data['feature']), 5);
      $category = Yaml::dump($data['category'], 5);
    }

    return [
      'tree' => [
        '#markup' => $result,
        '#attached' => ['library' => ['cmlapi/cmlapi.jstree']],
      ],
      'feature' => ['#markup' => "<pre>$feature</pre>"],
      'category' => ['#markup' => "<pre>$category</pre>"],
    ];
  }

  /**
   * Json import.
   */
  public function prepareFeature($features) {
    $result = [];
    if (!empty($features)) {
      foreach ($features as $feature) {
        $id = $feature['Ид'];
        $name = $feature['Наименование'];
        if ($feature['ТипЗначений'] == 'Справочник') {
          $sprav = [];
          if (isset($feature['ВариантыЗначений']['Справочник'])) {
            $sprav = XmlObject::xml2KeyVal($feature['ВариантыЗначений']['Справочник']);
          }
          $result['taxonomy']["$name($id)"] = $sprav;
        }
        else {
          $result['fields']["$name($id)"] = $feature['ТипЗначений'];
        }
      }
    }
    return $result;
  }

  /**
   * Json import.
   */
  public function data($cml) {
    $cml_xml = GetLastCml::load($cml)->field_file->getValue();
    $data = FALSE;
    if (!empty($cml_xml)) {
      foreach ($cml_xml as $xml) {
        $file = file_load($xml['target_id']);
        $filename = $file->getFilename();
        $filepath = drupal_realpath($file->getFileUri());
        $pos = strpos($filename, 'import');
        if ($pos === 0) {
          $xmlObj = new XmlObject();
          $xmlObj->parseXmlFile($filepath);
          $data = [
            'group' => CatalogParcer::parce($xmlObj->xmlString, FALSE),
            'feature' => CatalogParcer::parceFeature($xmlObj->xmlString),
            'category' => CatalogParcer::parceCategory($xmlObj->xmlString),
          ];
        }
      }
    }
    return $data;
  }

  /**
   * Render.
   */
  public function renderGroups($groups, $parent = FALSE) {
    $output = '<ul>';

    if (!empty($groups)) {
      $groups = XmlObject::arrayNormalize($groups);

      foreach ($groups as $group) {
        $data = FALSE;
        if ($parent) {
          $data = " data-jstree='{ \"opened\" : true }' ";
        }
        $output .= '<li ' . $data . '>';
        $output .= $group['Наименование'];
        if (!empty($group['Группы']['Группа'])) {
          $output .= self::renderGroups($group['Группы']['Группа']);
        }
        $output .= '</li>';
      }
      $output .= '</ul>';
    }
    return $output;
  }

}
